@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/signal.c
  *
- * Copyright (C) 2005-2009  NTT DATA CORPORATION
+ * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 1.7.1+   2009/12/20
+ * Version: 1.7.3+   2015/01/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -14,7 +14,7 @@
 
 /* To support PID namespace. */
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
-#define find_task_by_pid find_task_by_vpid
+#define find_task_by_pid ccsecurity_exports.find_task_by_vpid
 #endif
 
 /**
@@ -55,13 +55,14 @@ static int ccs_signal_acl2(const int sig, const int pid)
 	struct ccs_acl_info *ptr;
 	const u16 hash = sig;
 	int error;
-	if (ccs_init_request_info(&r, NULL, CCS_MAC_SIGNAL)
-	    == CCS_CONFIG_DISABLED)
+	const struct ccs_domain_info * const domain = ccs_current_domain();
+	if (ccs_init_request_info(&r, CCS_MAC_SIGNAL) == CCS_CONFIG_DISABLED)
 		return 0;
 	if (!sig)
 		return 0;                /* No check for NULL signal. */
-	if (sys_getpid() == pid) {
-		ccs_audit_signal_log(&r, sig, r.domain->domainname->name, true);
+	if (ccsecurity_exports.sys_getpid() == pid) {
+		ccs_audit_signal_log(&r, sig, domain->domainname->name,
+				     true);
 		return 0;                /* No check for self process. */
 	}
 	{ /* Simplified checking. */
@@ -81,14 +82,14 @@ static int ccs_signal_acl2(const int sig, const int pid)
 	}
 	if (!dest)
 		return 0; /* I can't find destinatioin. */
-	if (r.domain == dest) {
-		ccs_audit_signal_log(&r, sig, r.domain->domainname->name, true);
+	if (domain == dest) {
+		ccs_audit_signal_log(&r, sig, domain->domainname->name, true);
 		return 0;                /* No check for self domain. */
 	}
 	dest_pattern = dest->domainname->name;
 	do {
 		error = -EPERM;
-		list_for_each_entry_rcu(ptr, &r.domain->acl_info_list, list) {
+		list_for_each_entry_rcu(ptr, &domain->acl_info_list, list) {
 			struct ccs_signal_acl *acl;
 			if (ptr->is_deleted ||
 			    ptr->type != CCS_TYPE_SIGNAL_ACL)
@@ -116,7 +117,7 @@ static int ccs_signal_acl2(const int sig, const int pid)
 			break;
 		error = ccs_supervisor(&r, CCS_KEYWORD_ALLOW_SIGNAL "%d %s\n",
 				       sig, dest_pattern);
-	} while (error == 1);
+	} while (error == CCS_RETRY_REQUEST);
 	return error;
 }
 
@@ -150,7 +151,6 @@ int ccs_write_signal_policy(char *data, struct ccs_domain_info *domain,
 			    struct ccs_condition *condition,
 			    const bool is_delete)
 {
-	struct ccs_signal_acl *entry = NULL;
 	struct ccs_acl_info *ptr;
 	struct ccs_signal_acl e = { .head.type = CCS_TYPE_SIGNAL_ACL,
 				    .head.cond = condition };
@@ -164,9 +164,8 @@ int ccs_write_signal_policy(char *data, struct ccs_domain_info *domain,
 	e.domainname = ccs_get_name(domainname + 1);
 	if (!e.domainname)
 		return -ENOMEM;
-	if (!is_delete)
-		entry = kmalloc(sizeof(*entry), GFP_KERNEL);
-	mutex_lock(&ccs_policy_lock);
+	if (mutex_lock_interruptible(&ccs_policy_lock))
+		goto out;
 	list_for_each_entry_rcu(ptr, &domain->acl_info_list, list) {
 		struct ccs_signal_acl *acl =
 			container_of(ptr, struct ccs_signal_acl, head);
@@ -177,14 +176,16 @@ int ccs_write_signal_policy(char *data, struct ccs_domain_info *domain,
 		error = 0;
 		break;
 	}
-	if (!is_delete && error && ccs_commit_ok(entry, &e, sizeof(e))) {
-		ccs_add_domain_acl(domain, &entry->head);
-		entry = NULL;
-		error = 0;
+	if (!is_delete && error) {
+		struct ccs_signal_acl *entry = ccs_commit_ok(&e, sizeof(e));
+		if (entry) {
+			ccs_add_domain_acl(domain, &entry->head);
+			error = 0;
+		}
 	}
 	mutex_unlock(&ccs_policy_lock);
+ out:
 	ccs_put_name(e.domainname);
-	kfree(entry);
 	return error;
 }
 
@@ -196,7 +197,7 @@ int ccs_write_signal_policy(char *data, struct ccs_domain_info *domain,
  *
  * Returns 0 on success, negative value otherwise.
  */
-int ccs_kill_permission(pid_t pid, int sig)
+static int __ccs_kill_permission(pid_t pid, int sig)
 {
 	if (sig && (!ccs_capable(CCS_SYS_KILL) ||
 		    ccs_signal_acl(sig, pid)))
@@ -213,7 +214,7 @@ int ccs_kill_permission(pid_t pid, int sig)
  *
  * Returns 0 on success, negative value otherwise.
  */
-int ccs_tgkill_permission(pid_t tgid, pid_t pid, int sig)
+static int __ccs_tgkill_permission(pid_t tgid, pid_t pid, int sig)
 {
 	if (sig && (!ccs_capable(CCS_SYS_KILL) ||
 		    ccs_signal_acl(sig, pid)))
@@ -229,7 +230,7 @@ int ccs_tgkill_permission(pid_t tgid, pid_t pid, int sig)
  *
  * Returns 0 on success, negative value otherwise.
  */
-int ccs_tkill_permission(pid_t pid, int sig)
+static int __ccs_tkill_permission(pid_t pid, int sig)
 {
 	if (sig && (!ccs_capable(CCS_SYS_KILL) ||
 		    ccs_signal_acl(sig, pid)))
@@ -237,7 +238,7 @@ int ccs_tkill_permission(pid_t pid, int sig)
 	return 0;
 }
 
-int ccs_sigqueue_permission(pid_t pid, int sig)
+static int __ccs_sigqueue_permission(pid_t pid, int sig)
 {
 	if (sig && (!ccs_capable(CCS_SYS_KILL) ||
 		    ccs_signal_acl(sig, pid)))
@@ -245,10 +246,19 @@ int ccs_sigqueue_permission(pid_t pid, int sig)
 	return 0;
 }
 
-int ccs_tgsigqueue_permission(pid_t tgid, pid_t pid, int sig)
+static int __ccs_tgsigqueue_permission(pid_t tgid, pid_t pid, int sig)
 {
 	if (sig && (!ccs_capable(CCS_SYS_KILL) ||
 		    ccs_signal_acl(sig, pid)))
 		return -EPERM;
 	return 0;
 }
+
+void __init ccs_signal_init(void)
+{
+	ccsecurity_ops.kill_permission = __ccs_kill_permission;
+	ccsecurity_ops.tgkill_permission = __ccs_tgkill_permission;
+	ccsecurity_ops.tkill_permission = __ccs_tkill_permission;
+	ccsecurity_ops.sigqueue_permission = __ccs_sigqueue_permission;
+	ccsecurity_ops.tgsigqueue_permission = __ccs_tgsigqueue_permission;
+}
